bridge              =(require "./../shared/Bridge").bridge
utils               = require "./../shared/Utils"
{Constants}         = require "./../shared/Constants"
{TableComponent}    = require "./TableComponent"

# Ported from Vekter/src/document/components/utils/nodes.ts
defaultVekterName = (data) ->
    vekterClass = data.__framerInstanceInfo.vekterClass
    if vekterClass is "ImageNode"
        return data.__framerInstanceInfo.originalFilename || "Image"
    if vekterClass is "SVGNode"
        return data.__framerInstanceInfo.originalFilename || "Icon"
    if vekterClass is "FrameNode"
        return "Artboard"
    if vekterClass is "TextNode"
        return data.__framerInstanceInfo.text.replace(/\s+/g, " ")
    if vekterClass is "VectorNode"
        return "Vector"
    if vekterClass is "OvalNode"
        return "Oval"
    return "Rectangle"

displayName = (rowData) ->
    if rowData.__framerInstanceInfo.targetName?
        return rowData.__framerInstanceInfo.targetName
    if rowData.name? and rowData.name.length > 0
        return rowData.name
    if rowData.__framerInstanceInfo.vekterClass?
        return defaultVekterName(rowData)
    if rowData.__framerInstanceInfo.name?
        return rowData.__framerInstanceInfo.name
    return "#{rowData.__framerInstanceInfo.framerClass} #{rowData.id}"


class exports.LayerPanel extends TableComponent

    constructor: (options = {}) ->

        options.name ?= "LayerPanel"

        @messages = Constants.messages.layerpanel
        @data = []
        super options
        @render()

        framerViewMessages = Constants.messages.framerview
        framerStudioMessages = Constants.messages.studio

        bridge.on framerViewMessages.layers, @setData
        bridge.on framerStudioMessages.state, @handleStudioState
        bridge.on framerStudioMessages.layerPanelEnter, @handleStudioLayerPanelEnter
        bridge.on framerStudioMessages.layerPanelLeave, @handleStudioLayerPanelLeave

        @bumper.on Events.MouseOver, =>
            if @_highlightedLayerId?
                @unhighlightRow()
                bridge.send(@messages.itemRollOut, {})

    setData: (data) =>
        # We need to check if the current highlighted layer is still existing.
        # If not we need to reset the highlighter.
        if @_highlightedLayerId and @_highlightedLayerId not in _.pluck(data, "id")
            bridge.send(@messages.reset, {})

        # Sort the layers by hierarchy and render the table view again
        @data = utils.sortLayersByHierarchy(data)
        @render()

    totalRows: -> @data.length
    heightForRow: -> Constants.sizes.rowHeight

    marginForRow: -> 0

    createRow: ->
        row = new Layer
            name: "row"
            height: @heightForRow()
            width: 212

        row.style =
            borderBottom: "1px solid #{Constants.colors.border}"

        # Indentation layer (for the hierarchy)
        indent = Constants.sizes.paddingLeft + Constants.sizes.indentOffset
        row.indentLayer = new Layer
            name: "indent"
            x: indent
            width: row.width - indent
            height: Constants.sizes.indentHeight
            parent: row

        row.indentLayer.style =
            background: "repeating-linear-gradient(to right,
                    #{Constants.colors.indent},
                    #{Constants.colors.indent} #{Constants.sizes.indentWidth}px,
                    #{Constants.colors.background} #{Constants.sizes.indentWidth}px,
                    #{Constants.colors.background} #{Constants.sizes.indent}px
                )"
        row.indentLayer.centerY()

        # Text layer (containing the name)
        row.textLayer = new Layer
            name: "text"
            width: row.width
            height: @heightForRow()
            parent: row
            backgroundColor: Constants.colors.background
            clip: true

        row.textLayer.style =
            font: "11px -apple-system, Helvetica Neue, Helvetica"
            color: Constants.colors.text
            lineHeight: "#{@heightForRow()}px"
            WebkitUserSelect: "none"
            WebkitFontSmoothing: "subpixel-antialiased"

        row.targetLayer = new Layer
            name: "target"
            parent: row
            x: Align.right(-5)
            y: Align.center
            size: 14
            backgroundColor: null
            visible: false
            html: '<svg width="14" height="14" viewBox="0 0 14 14" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><title>icon-target-active-selected</title><defs><circle id="a" cx="6.5" cy="6.5" r="5.5"/><circle id="b" cx="6.5" cy="6.5" r="2.5"/></defs><g fill="none" fill-rule="evenodd"><use fill-opacity=".3" fill="#FFF" xlink:href="#a"/><circle stroke="#FFF" cx="6.5" cy="6.5" r="5"/><use fill-opacity=".7" fill="#FFF" xlink:href="#b"/><circle stroke="#FFF" cx="6.5" cy="6.5" r="2"/></g></svg>'

        row.targetLayer.states =
            target:
                visible: true
        
        row.targetLayer.onClick (event) =>
            if row.targetLayer.visible
                bridge.send(@messages.itemContextMenu,
                    id: row.data.id
                    data: row.data.__framerInstanceInfo
                    location: {x: event.pageX, y: event.pageY}
                )

        # Events
        row.onMouseOver @handleMouseOver
        row.onMouseDown @handleClick
        row._element.addEventListener(
            "contextmenu",
            @handleContextMenu(row)
            false)
        row

    updateRow: (index, row) ->
        row.data = @data[index]

        row.textLayer.html = "<div style='overflow: hidden;text-overflow:ellipsis;white-space:nowrap;'>#{displayName(row.data)}</div>"
        if row.data.__framerInstanceInfo.targetName?
            row.targetLayer.stateSwitch "target"
        else
            row.targetLayer.stateSwitch "default"

        indent = Constants.sizes.paddingLeft + (row.data.level * Constants.sizes.indent)

        if row.data.level is 0
            row.indentLayer.style.background = "none"

        row.indentLayer.width = indent
        row.textLayer.x = indent
        row.textLayer.width = row.width - indent - 24

        if @isRowSelected("hover", row)
            row.textLayer.style.color = Constants.colors.textHighlight
        else
            row.textLayer.style.color = @_textColor(row.data)

        color = if @isRowSelected("selection", row)
            Constants.colors.tableSelectionBackground
        else
            Constants.colors.background

        row.textLayer.backgroundColor = color
        row.backgroundColor = color

    handleMouseOver: (event, row) =>
        @_highlightedLayerId = row?.data?.id
        @sendItemRollOver row

    handleClick: (event, row) =>
        if event.metaKey
            bridge.send(@messages.itemCmdClick,
                hash: row.data.__framerInstanceInfo?.hash
                state: row.data.currentState
            )
        else
            bridge.send(@messages.itemClick,
                id: row.data.id
                data: row.data.__framerInstanceInfo
            )

    handleContextMenu: (row) =>
        return (event) =>
            bridge.send(@messages.itemContextMenu,
                id: row.data.id
                data: row.data.__framerInstanceInfo
                location: {x: event.pageX, y: event.pageY}
            )

    handleStudioState: (state) =>

        console.log "layerPanel:handleStudioState", state

        if @_highlightedLayerId == state.highlightingTarget?.runtimeId
            return

        id = state.highlightingTarget?.runtimeId
        if id?
            for row in @_rowLayers
                if row.data.id == id
                    @highlightRow row
                    return

        @unhighlightRow()

    highlightRow: (row) ->
        # @setSelectedRows("hover", [row])
        @_highlightedLayerId = row.data.id

    unhighlightRow: ->
        # @setSelectedRows("hover", [])
        @_highlightedLayerId = null

    handleScrollerResize: (width, height) =>
        @width = width

    handleStudioLayerPanelEnter: =>

        console.log("layerPanel:handleStudioLayerPanelEnter")

        @entered = true
        @delay = Utils.delay .25, =>
            clearTimeout @delay
            @delay = null
            @sendItemRollOver @delayedRow if @delayedRow?
            @delayedRow = null

    handleStudioLayerPanelLeave: =>

        console.log("layerPanel:handleStudioLayerPanelLeave")

        clearTimeout @delay if @delay?
        @delayedRow = null
        @delay = null
        @entered = false

    sendItemRollOver: (row) =>
        if @entered and not @delay?
            console.log "sentItemRollOver"
            @highlightRow row
            bridge.send(@messages.itemRollOver, {id:row.data.id, data: row.data.__framerInstanceInfo})
        else
            @delayedRow = row

    # Internals
    #

    _textColor: (data) ->
        return Constants.colors.target if data.__framerInstanceInfo.targetName? or data.__framerInstanceInfo.location?
        return Constants.colors.text if data.visibleResult
        return Constants.colors.textInvisible
