{bridge} = require "./Bridge"

traverseUp = (layer) ->
	layers = [layer]
	while layer.parent
		layers.push(layer.parent)
		layer = layer.parent
	return layers

getLayerProperties = (layer) ->
	properties =
		id: layer.id
		name: layer.name
		parent: null

	if layer.__framerInstanceInfo?
		properties.__framerInstanceInfo = _.clone(layer.__framerInstanceInfo)
	else
		properties.__framerInstanceInfo = hash: "#il|#{layer.__framerImportedFromPath}|#{properties.name}"
	properties.__framerInstanceInfo.framerClass = layer.constructor.name

	_.extend(properties, _.pick(layer, [
		"x", "y", "z",
		"index", "width", "height",
		"scale", "opacity",
		"rotationX", "rotationY", "rotationZ",
		"blur"
	]))

	if properties.visible is false
		properties.visibleResult = false
	else
		properties.visibleResult = false not in _.pluck(traverseUp(layer), "visible")

	properties.parent = layer.parent.id if layer.parent?
	properties.currentState = layer.states.current.name

	return properties

ContextListenerPropertyUpdateKeys = ["name", "parent", "index", "visible"]


class ContextListener

	constructor: (context) ->
		@_context = context

		@update = _.debounce(@_update, 10)

		@_context.on("reset", @onContextReset)
		@_context.on("layer:create", @onLayerCreate)
		@_context.on("layer:destroy", @onLayerDestroy)

	_update: =>
		# We need this for backwards compatibility
		layers = @_context._layers or @_context._layerList
		bridge.send("framerView:layers", _.map(@_context._layers, getLayerProperties))

	send: (eventName, info) ->
		window._bridge("ui:#{eventName}", info)

	onContextReset: =>
		@update()

	onLayerCreate: (layer) =>
		@update()
		for key in ContextListenerPropertyUpdateKeys
			layer.on("change:#{key}", @update)

	onLayerDestroy: (layer) =>
		@update()
		for key in ContextListenerPropertyUpdateKeys
			layer.off("change:#{key}", @update)

exports.ContextListener = ContextListener
